% MTODO 3 
% TEST 3-1
%
% La referencia utilizada ha sido:
%
%	J. S. Cnovas, A. Guillamn
%	PERMUTATIONS AND TIME SERIES ANALYSIS
%	Chaos 19 (2009), 043103
%
% Los pasos que hay que seguir para hacer el test son:
% 
% PASO 0: Se introducen los parmetros de entrada de la funcin, que
% son la serie temporal, la dimensin de inmersin y el nivel de confianza.
%
% PASO 1: Clculos previos.
% PASO 2: Calculamos el estadstico de contraste, que es el nmero de
% permutaciones diferentes que aparecen en la serie.
% PASO 3: Realizamos el contraste de hiptesis.

function t3_1=test_3_1(serie_temp,m,nivel_confianza)

% PASO 1
% CLCULOS PREVIOS

serie=serie_temp; % serie temporal
T=length(serie); % nmero de elementos de la serie
k=fix(T/m); % nmero de tramos

% PASO 2
% CALCULAMOS EL ESTADSTICO DE CONTRASTE, QUE ES EL NMERO DE PERMUTACIONES
% DIFERENTES QUE APARECEN EN LA SERIE

% Dividimos la serie temporal en tramos siguiendo los siguientes puntos:
%
%   1) El primer tramo empieza en el primer elemento de la serie temporal.
%   2) Los tramos contienen todos la misma cantidad de elementos, que viene
%   marcada por la dimensin de inmersin m.
%   3) Los tramos se construyen tomando elementos consecutivos.
%   4) No hay solapamiento entre tramos consecutivos, ni tampoco elementos
%   libres entre ellos.
%

% A los tramos se les hace corresponder una permutacin que indique el
% orden de los elementos del mismo.

% Determinamos el nmero de permutaciones distintas que aparecen en la
% serie temporal.

M_permutaciones=zeros(factorial(m),m);

trozo_serie1=serie(1:m);
[trozo_ordenado1,permutacion1]=sort(trozo_serie1);
M_permutaciones(1,:)=permutacion1;
Npermutaciones_diferentes=1;

for cont1=m+1:m:T-m+1
    
    trozo_serie=serie(cont1:cont1+m-1);
    [trozo_ordenado,permutacion]=sort(trozo_serie);
        
    testigo=1;
    
    for cont2=1:Npermutaciones_diferentes
        
        if isequal(M_permutaciones(cont2,:),permutacion)
            
            testigo=0;
            break            
            
        end
        
    end
    
    if testigo
        
    Npermutaciones_diferentes=Npermutaciones_diferentes+1;    
    M_permutaciones(Npermutaciones_diferentes,:)=permutacion;
    
    end
    
end

% Y ya tenemos el estadstico de contraste, que es simplemente la cantidad
% de permutaciones distintas que hay en la serie temporal.

Estadistico_contraste=Npermutaciones_diferentes; % estadstico de contraste

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

% Con el fin de determinar el valor crtico, necesario para realizar el
% contraste de hiptesis, hay que recurrir a la funcin de distribucin de
% probabilidad.
% Pero los valores crticos para todos los posibles tamaos de la serie ya
% han sido previamente calculados y almacenados en un fichero.
% De modo que slo hay que leer el valor necesario del fichero
% correspondiente.

valores_criticos=load(strcat('VECTORm',num2str(m),'nc',num2str(nivel_confianza),'.txt'));
T_valores_criticos=length(valores_criticos);

if k>T_valores_criticos
    
    valor_critico=valores_criticos(T_valores_limite);
    
else

    valor_critico=valores_criticos(k);
    
end

% No rechazamos la hiptesis nula si el estadstico de contraste es mayor o
% igual al valor crtico; en esta situacin t3_1 toma el valor 1.
% Si el estadstico de contraste es menor que el valor crtico, entonces se
% rechaza la hiptesis nula; aqu t3_1 tomara el valor 0.

if Estadistico_contraste>=valor_critico
    
    t3_1=1;
    
else
        
    t3_1=0;
        
end
