% MTODO 3 
% TEST 3-2
%
% La referencia utilizada ha sido:
%
%	J. S. Cnovas, A. Guillamn
%	PERMUTATIONS AND TIME SERIES ANALYSIS
%	Chaos 19 (2009), 043103
%
% Seguimos los siguientes pasos para hacer el test:
%
% PASO 0: Se introducen los parmetros de entrada de la funcin, que
% son la serie temporal, la dimensin de inmersin y el nivel de confianza.
% 
% PASO 1: Clculos previos.
% PASO 2: Calculamos el estadstico de contraste.
% PASO 3: Realizamos el contraste de hiptesis.

function t3_2=test_3_2(serie_temp,m,nivel_confianza)

% PASO 1
% CLCULOS PREVIOS

serie=serie_temp; % serie temporal
T=length(serie); % nmero de elementos de la serie
nivel_significacion=1-nivel_confianza/100; % nivel de significacin

% PASO 2
% CALCULAMOS EL ESTADSTICO DE CONTRASTE

% Vamos a dividir la serie temporal en varias partes (ventanas).
% La primera ventana se construye desde el primer elemento de la serie.
% Se har de tal manera que, entre ellas, no haya solapamientos ni
% elementos libres.
% El tamao de las ventanas se tomar igual a m*m!.
% En caso de que slo se pueda tomar una ventana, sta abarcar toda la
% serie temporal; as se evita una gran prdida de elementos.

% De los criterios para la divisin de la serie en ventanas se deduce que,
% en caso de haber elementos sobrantes, estos sern los del final de la
% serie.

if(T>=m*factorial(m)*2)
    
    Tventana=m*factorial(m);
    Nventanas=fix(T/Tventana);
    
else
    
    Tventana=T;
    Nventanas=1;
    
end

% En cada ventana hacemos una subdivisin en tramos, cumpliendo:
% 
%   1) Comenzamos la subdivisin desde el primer elemento de la ventana.
%   2) Cada tramo abarca el mismo nmero de elementos, que es igual a
%   la dimensin de inmersin m.
%   3) Los elementos que incluye cada trama son elementos consecutivos.
%   4) El desplazamiento que hay entre dos tramos consecutivos es de m
%   elementos, no existiendo solapamiento entre ellos.
%

% A todos los tramos se les hace corresponder una permutacin que marca el
% orden de sus elementos.

% Ahora se va a determinar el nmero de permutaciones diferentes que hay en
% cada una de las ventanas en que se ha dividido la serie.

k=Tventana/m; % nmero de tramos en que se divide cada ventana
Nmax_permutaciones_diferentes=min(fix(k),factorial(m)); % nmero mximo de permutaciones diferentes en cada ventana

% Introducimos un vector en cuyos elementos se almacenen el nmero de
% permutaciones diferentes de cada ventana.

v_Npermutaciones_diferentes=zeros(Nventanas,1);

for cont1=1:Nventanas
    
    ventana=serie((cont1-1)*Tventana+1:cont1*Tventana);
    M_permutaciones=zeros(factorial(m),m);

    trozo_ventana1=ventana(1:m);
    [trozo_ordenado1,permutacion1]=sort(trozo_ventana1);
    M_permutaciones(1,:)=permutacion1;
    Npermutaciones_diferentes=1;
    
    for cont2=m+1:m:Tventana-m+1
    
        trozo_ventana=ventana(cont2:cont2+m-1);
        [trozo_ordenado,permutacion]=sort(trozo_ventana);
        
        testigo=1;
    
        for cont3=1:Npermutaciones_diferentes
        
            if isequal(M_permutaciones(cont3,:),permutacion)
            
                testigo=0;
                break            
            
            end
        
        end
    
        if testigo
        
        Npermutaciones_diferentes=Npermutaciones_diferentes+1;    
        M_permutaciones(Npermutaciones_diferentes,:)=permutacion;
    
        end
    
    end

    v_Npermutaciones_diferentes(cont1)=Npermutaciones_diferentes;

end

% Calculamos ya el estadstico de contraste.
% Para ello es necesario disponer de la funcin de probabilidad.
% La funcin de probabilidad ha sido calculada previamente; sus valores
% estn almacenados en un fichero externo, y nicamente tenemos que
% tomarlos del mismo.

probabilidad=load(strcat('VPROB_T',num2str(Tventana),'m',num2str(m),'.txt'));
probabilidad=probabilidad';

% El clculo del estadstico de contraste se lleva a cabo de la siguiente
% forma:
%
%                             min{k,m!}
%       ESTADSTICO_CONTRASTE=   SUM   (f_i-e_i)^2/e_i
%                                i=1   
%
% donde f_i representa el nmero de ventanas con exactamente i
% permutaciones diferentes; y e_i es la frecuencia esperada, es decir,
% e_i=n*Pr(W=i), siendo n el nmero de ventanas y W la variable aleatoria
% que indica el nmero de permutaciones diferentes.

Estadistico_contraste=0; % estadistco de contraste

for cont4=1:Nmax_permutaciones_diferentes
    
    fi=0;
    ei=Nventanas*probabilidad(cont4);
    
    for cont5=1:Nventanas
        
        if v_Npermutaciones_diferentes(cont5)==cont4
            
            fi=fi+1;
            
        end
        
    end
    
    Estadistico_contraste=Estadistico_contraste+((fi-ei)^2/ei);
    
end

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

grados_libertad=Nmax_permutaciones_diferentes-1; % grados de libertad
chi_cuadrado=chi2inv(1-nivel_significacion,grados_libertad); % valor crtico

% Aceptamos la hiptesis nula siempre que el estadstico de contraste sea
% menor o igual al valor crtico; en dicho caso la variable t3_2 adopta el
% valor unidad.
% Rechazamos la hiptesis nula en caso contrario; situacin en la que t3_2
% toma el valor nulo.

if Estadistico_contraste<=chi_cuadrado
    
    t3_2=1;
    
else
        
    t3_2=0;
        
end
